/////////////////////////////////////////////////////////////////////////////////
//                  CONFIDENTIAL AND PROPRIETARY INFORMATION
//             (C) 2003 Synopsys Inc (Unpublished) and Chris Spear
//                             All Rights Reserved
//
// This file can be freely distributed providing this (C) notice remains
/////////////////////////////////////////////////////////////////////////////////

/* Module containing calls to distribution functions */
module dist;
   task dist_uniform;
      inout [31:0] seed;
      input [31:0] start;
      input [31:0] endi;
      output [31:0] dist;
      dist = $dist_uniform(seed, start, endi);
   endtask // dist_uniform

   task dist_normal;
      inout [31:0] seed;
      input [31:0] mean;
      input [31:0] sdev;
      output [31:0] dist;
      dist = $dist_normal(seed, mean, sdev);
   endtask // dist_normal

   task dist_exponential;
      inout [31:0] seed;
      input [31:0] mean;
      output [31:0] dist;
      dist = $dist_exponential(seed, mean);
   endtask // dist_exponential

   task dist_poisson;
      inout [31:0] seed;
      input [31:0] mean;
      output [31:0] dist;
      dist = $dist_poisson(seed, mean);
   endtask // dist_poisson


   // Generate a "bathtub" type distribution, high on
   // the sides, low in the middle
   task dist_bathtub;
      inout [31:0] seed;
      input [31:0] mean, lo, hi;
      output [31:0] dist;
      reg [31:0] j, size;
      begin
	 // Check ranges and fix if needed
	 if (lo > hi)
	   begin
	      j = lo;
	      lo = hi;
	      hi = j;
	   end
	 
	 // Get a value in the proper range
	 size = hi - lo + 1;
	 j = $dist_exponential(seed, mean);
	 while (j >= size)
	   j = $dist_exponential(seed, mean);
	 
	 // Put this on the left or right side of the tub
	 if ($random()&1)
	   dist = j + lo;
	 else
	   dist = hi - j;
	 //$display("VCS exp returned %0d, returning %0d", j, dist);
      end
   endtask // dist_bathtub
   
endmodule // dist
